==> libgoose-0.0.1.tar.gz: libgoose-0.0.1/src/regression.cc <==
// This is -*- C++ -*-
// $Id: regression.cc,v 1.2 1998/08/20 22:03:58 trow Exp $

/* 
 * Regression.cc
 *
 * Copyright (C) 1998 EMC Capital Management, Inc.
 *
 * Developed by Jon Trowbridge <trow@emccta.com>.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */


#include "matrix.h"
#include "regression.h"

void
LinearRegression::model(const DataSet& x, const DataSet& y)
{
  if (x.size() != y.size()) {
    slope_ = intercept_ = corr_ = 0;
    return;
  }

  slope_ = x.covar(y) / x.var();
  intercept_ = y.mean() - slope_ * x.mean();
  corr_ = slope_ * x.sdev() / y.sdev();
}

DataSet*
LinearRegression::residuals(const DataSet& x, const DataSet& y)
{
  if (x.size() != y.size())
    return 0;

  DataSet* res = new DataSet;
  res->reserve(x.size());

  for(size_t i=0; i<x.size(); ++i)
    res->add(y.data(i) - predict(x.data(i)));

  return res;
}

/////////////////////////////////////////////////////////////////////////////

void
PolynomialRegression::model(const DataSet& x, const DataSet& y, int deg)
{
  delete [] coeff_;
  coeff_ = 0;
  degree_ = -1;
  if (x.size() != y.size()) return;

  degree_ = deg;
  coeff_ = new double[degree_+1];

  double* vec = new double[degree_+1];
  for(int i=0; i<=degree_; ++i) vec[i] = coeff_[i] = 0;
  
  for(size_t i=0; i<x.size(); ++i) {
    double xx = x.data(i);
    double yy = y.data(i);
    double q = 1;
    for(int j=0; j<=degree_; ++j) {
      vec[j] += q * yy;
      q *= xx;
    }
  }

  int NN = (degree_+1)*(degree_+1);
  double* mat = new double[NN];
  for(int i=0; i<NN; ++i) mat[i] = 0;

  for(size_t k=0; k<=x.size(); ++k) {
    double xk = x.data(k);
    double xki = 1;
    for(int i=0; i<=degree_; ++i) {
      double xkij = xki;
      int ii = i*(degree_+1);
      for(int j=0; j<=degree_; ++j) {
	mat[ii+j] += xkij;
	xkij *= xk;
      }
      xki *= xk;
    }
  }

  if (! matrix_invert(mat, degree_+1)) 
    cerr << "Singular matrix!  Uh oh!" << endl;
 
  for(int i=0; i<=degree_; ++i) {
    double sum=0;
    int ii=i*(degree_+1);
    for(int j=0; j<=degree_; ++j)
      sum += vec[j] * mat[ii+j];
    coeff_[i] = sum;
  }

  delete [] vec;
  delete [] mat;
}


// $Id: regression.cc,v 1.2 1998/08/20 22:03:58 trow Exp $

==> libgoose-0.0.2.tar.gz: libgoose-0.0.2/src/regression.cc <==
// This is -*- C++ -*-
// $Id: regression.cc,v 1.4 1998/09/14 04:48:38 trow Exp $

/* 
 * Regression.cc
 *
 * Copyright (C) 1998 EMC Capital Management, Inc.
 *
 * Developed by Jon Trowbridge <trow@emccta.com>.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */


#include "matrix.h"
#include "regression.h"

void
LinearRegression::model(const DataSet& x, const DataSet& y)
{
  if (x.size() != y.size()) {
    slope_ = intercept_ = corr_ = 0;
    return;
  }

  slope_ = x.covar(y) / x.var();
  intercept_ = y.mean() - slope_ * x.mean();
  corr_ = slope_ * x.sdev() / y.sdev();

  // Calculate residuals
  residuals_.clear();
  residuals_.reserve(x.size());
  for(size_t i=0; i<x.size(); ++i)
    residuals_.add(y.data(i) - predict(x.data(i)));
}

void
PolynomialRegression::model(const DataSet& x, const DataSet& y, int deg)
{
  delete [] coeff_;
  coeff_ = 0;
  degree_ = -1;
  if (x.size() != y.size()) return;

  degree_ = deg;
  coeff_ = new double[degree_+1];

  double* vec = new double[degree_+1];
  for(int i=0; i<=degree_; ++i) vec[i] = coeff_[i] = 0;
  
  for(size_t i=0; i<x.size(); ++i) {
    double xx = x.data(i);
    double yy = y.data(i);
    double q = 1;
    for(int j=0; j<=degree_; ++j) {
      vec[j] += q * yy;
      q *= xx;
    }
  }

  int NN = (degree_+1)*(degree_+1);
  double* mat = new double[NN];
  for(int i=0; i<NN; ++i) mat[i] = 0;

  for(size_t k=0; k<=x.size(); ++k) {
    double xk = x.data(k);
    double xki = 1;
    for(int i=0; i<=degree_; ++i) {
      double xkij = xki;
      int ii = i*(degree_+1);
      for(int j=0; j<=degree_; ++j) {
	mat[ii+j] += xkij;
	xkij *= xk;
      }
      xki *= xk;
    }
  }

  if (! matrix_invert(mat, degree_+1)) 
    cerr << "Singular matrix!  Uh oh!" << endl;
 
  for(int i=0; i<=degree_; ++i) {
    double sum=0;
    int ii=i*(degree_+1);
    for(int j=0; j<=degree_; ++j)
      sum += vec[j] * mat[ii+j];
    coeff_[i] = sum;
  }

  delete [] vec;
  delete [] mat;

  // Calculate residuals
  residuals_.clear();
  residuals_.reserve(x.size());
  for(size_t i=0; i<x.size(); ++i)
    residuals_.add(y.data(i) - predict(x.data(i)));
}


// $Id: regression.cc,v 1.4 1998/09/14 04:48:38 trow Exp $
